package org.owasp.csrfguard.http;

import java.io.IOException;
import java.io.PrintWriter;
import java.io.ByteArrayOutputStream;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpServletResponseWrapper;

public final class MutableHttpResponse extends HttpServletResponseWrapper
{
	private HttpServletResponse originalResponse = null;

	private ByteArrayOutputStream output = null;

	private FilterOutputStream stream = null;
	
	private PrintWriter writer = null;

	private int contentLength = -1;

	public MutableHttpResponse(HttpServletResponse originalResponse)
	{
		super(originalResponse);

		this.originalResponse = originalResponse;
		this.output = new ByteArrayOutputStream();
		this.stream = new FilterOutputStream(output);
		this.writer = new PrintWriter(stream);
	}
	
	public void flushBuffer() throws IOException
	{
		writer.flush();
		output.flush();
		stream.flush();
	}

	public int getBufferSize()
	{
		return output.size();
	}

	public ServletOutputStream getOutputStream()
	{
		return stream;
	}

	public PrintWriter getWriter() throws IOException
	{
		return writer;
	}

	public boolean isCommitted()
	{
		return false;
	}

	public void reset() throws IllegalStateException
	{
		/** Clear any buffers * */
		output = new ByteArrayOutputStream();
		stream = new FilterOutputStream(output);
		writer = new PrintWriter(stream);
	}

	public void resetBuffer() throws IllegalStateException
	{
		/** Clear any buffers * */
		output = new ByteArrayOutputStream();
		stream = new FilterOutputStream(output);
		writer = new PrintWriter(stream);
	}

	public void setContentLength(int length)
	{
		this.contentLength = length;
		super.setContentLength(length);
	}
	
	public byte[] getContent() throws IOException
	{
		/**
		 * FIXME: Why do we have to flush here? Why isnt flushBuffer() getting
		 * called ?
		 */
		writer.flush();
		stream.flush();
		output.flush();
		
		return output.toByteArray();
	}

	public void setContent(String s) throws IOException
	{
		/** Clear any buffers * */
		output = new ByteArrayOutputStream();
		stream = new FilterOutputStream(output);
		writer = new PrintWriter(stream);

		output.write(s.getBytes());
	}

	public int getContentLength()
	{
		return contentLength;
	}

	public HttpServletResponse getOriginalResponse()
	{
		return originalResponse;
	}

	public FilterOutputStream getStream()
	{
		return stream;
	}

	public ByteArrayOutputStream getOutput()
	{
		return output;
	}

	public HttpServletResponse getResponse()
	{
		return this;
	}
}
